indexing

	description:

		"Variable resolvers"

	library: "Gobo Eiffel Ant"
	copyright: "Copyright (c) 2004, Sven Ehrke and others"
	license: "Eiffel Forum License v2 (see forum.txt)"
	date: "$Date: 2004/11/15 19:19:41 $"
	revision: "$Revision: 1.2 $"

class GEANT_PROJECT_VARIABLE_RESOLVER

inherit

	GEANT_VARIABLE_RESOLVER

	GEANT_SHARED_PROPERTIES
		export {NONE} all end

	KL_SHARED_FILE_SYSTEM
		export {NONE} all end

creation

	make

feature {NONE} -- Initialization

	make is
			-- Create new Project Variable resolver.
		do
		end

feature -- Access

	value (a_name: STRING): STRING is
			-- Value of variable `a_name' as a result of searching in
			-- commandline variables, project variables, environment variables
			-- in this order; Void if not found
		do
				-- Check non overrridable variables:
			if STRING_.same_string (a_name, "cwd") then
				Result := file_system.cwd
			end

			if Result = Void then
					-- Search commandline variables:
				Commandline_variables.search (a_name)
				if Commandline_variables.found then
					Result := Commandline_variables.found_item
				end
			end

			if Result = Void then
					-- Search project variables:
				variables.search (a_name)
				if variables.found then
					Result := variables.found_item
				end
			end

			if Result = Void then
					-- Search environment variables:
				if Execution_environment.variable_value (a_name) /= Void then
					Result := Execution_environment.variable_value (a_name)
				end
			end

		end

	boolean_condition_value (a_condition: STRING): BOOLEAN is
			-- Is `a_condition' True?;
			-- used for "if" and "unless" attributes;
			-- possible forms:
			-- "$foo": True if variable `foo' is defined
			-- "${foo}": True if variable `foo' is defined
			-- "$foo=bar" | "${foo}=bar" | "bar=$foo" | "bar=${foo}":
			--       True if variable `foo' is defined and its value is "bar"
			-- if `a_condition' is not in either form Result is `False'
		require
			condition_not_void: a_condition /= Void
		local
			a_tokens: DS_ARRAYED_LIST [STRING]
			s: STRING
			s2: STRING
			a_string_interpreter: GEANT_STRING_INTERPRETER
		do
			a_tokens := string_tokens (a_condition, '=')
			if a_tokens.count = 1 then
					-- `a_condition' should be in form "$foo";
					-- Check if $foo is defined.
				s := a_tokens.item (1)
				if s.count > 3 and then (s.item (1) = '$' and s.item (2) = '{' and s.item (s.count) = '}') then
						-- Handle "${bar}" form:
					s := s.substring (3, s.count - 1)
					Result := has (s)
				elseif s.count > 1 and then (s.item (1) = '$' and s.item (2) /= '{') then
						-- Handle "$bar" form:
					s := s.substring (2, s.count)
					Result := has (s)
				else
					exit_application (1, <<"geant: incorrect conditional: '", a_condition, "%'">>)
				end
			elseif a_tokens.count = 2 then
					-- `a_condition' should be in form "$foo=bar";
					-- Check if $foo equals "bar".
					-- TODO: Do this more efficiently:
				create a_string_interpreter.make
				a_string_interpreter.set_variable_resolver (Current)

				s := a_tokens.item (1)
				s := a_string_interpreter.interpreted_string (s)
				s2 := a_tokens.item (2)
				s2 := a_string_interpreter.interpreted_string (s2)
				Result := s.is_equal (s2)
			else
				exit_application (1, <<"geant: incorrect conditional: '", a_condition, "%'">>)
			end
		end

	variables: GEANT_VARIABLES
			-- Variables used for resolving

feature -- Status report

	has (a_name: STRING): BOOLEAN is
			-- Is there a variable named `a_name' in
			-- commandline variables, project variables, environment variables
			-- (searched in this order)?
		local
			a_value: STRING
		do
				-- Check non overrridable variables:
			Result := STRING_.same_string (a_name, "cwd")

			if not Result then
					-- Search commandline variables:
				Commandline_variables.search (a_name)
				Result := Commandline_variables.found
			end

			if not Result then
					-- Search project variables:
				variables.search (a_name)
				Result := variables.found
			end

			if not Result then
					-- Search environment variables:
				a_value := Execution_environment.variable_value (a_name)
				Result := a_value /= Void
			end
		end

feature -- Setting

	set_variables (a_variables: like variables) is
			-- Set `variables' to `a_variables'.
		require
			a_variables_not_void: a_variables /= Void
		do
			variables := a_variables
		ensure
			variables_set: variables = a_variables
		end

end
